% Calculation of the mean LOD of 365 solar (or sideral) days from given point of the orbit (aphelion,equinox, etc.)
% Toolkit: http://naif.jpl.nasa.gov/pub/naif/toolkit_docs/MATLAB/
% Tested : Matlab R2011b on Windows XP SP3
%     By : Poltavsky Sasha          10-19-2013
%    URL : www.bible-exodus.eu.pn/articles/astro_ephemeris/jpl_ephemeris/jpl_ephemeris.html#get_kernel_data
%------------------------------------------------------------------------------------------------------------------

function mean_LOD_from_365_solar_days
clear all; format short     
%**************************  GRAPHICAL INTERFACE  ************************************************
global g_s_p_o,             % select the begining point at the orbit
global g_s_p_d;             % select the begining point in day
global g_w_of_d;            % the way of determination the same moment in day
global path_de;             % the path to ephemeris files
global filename_de;         % the ephemeris file name
global save_path;
global g_s_et;              % the start point of the period
global g_e_et;              % the end point of the period
global g_erm;               % select the rotation model IAU of IRTF93
global g_abrc;              % select abberation correction
global g_obsrvr_pl;         % observer's place
global g_obsrvr_places;     % list of prepared observer's places
global g_obsrvr_lon;        
global g_obsrvr_lat;
global g_obsrvr_alt;
global g_lon_lat_alt;
global g_fl_mat;            % flag - to allow output into *.mat
global g_fl_xls;            % flag - to allow output into *.xls
global g_fl_fig;            % flag - to allow output into a graph plot
global g_fl_fig_save;       % flag - to save a graph plot
global g_fl_cf;             % flag - to allow output into q graph cftool

cspice_furnsh( 'd:\Documents and Settings\user\ \MATLAB\astro\mice\data\standard.tm' );  
[g_obsrvr_places, ~] = cspice_gcpool( 'NAIF_BODY_NAME', 1, 100 );

cspice_pcpool('TKFRAME_EARTH_FIXED_SPEC', 'MATRIX'); % preparing for selecting Earth rotation model IAU or ITRF93
cspice_pipool('TKFRAME_EARTH_FIXED_MATRIX', [ 1;  0;  0;  0;  1;  0;  0;   0;   1 ]);
    % PathName_work ='d:\Documents and Settings\user\ \MATLAB\astro\mice\test\time\mean LOD\';
%save_path = 'd:\Documents and Settings\user\ \MATLAB\astro\mice\test\time\mean LOD\';

      
keys={'','','path_de','','d:\Documents and Settings\user\ \MATLAB\astro\mice\data\';
       '','','filename_de','','de422.bsp';
       '','','save_path','','';
       '','','g_s_p_o', 'i', '1';
       '','','g_s_p_d', 'i', '1';
       '','','g_w_of_d', 'i', '1';
       '','','g_s_et', '', '01-01-1970 AD';
       '','','g_e_et', '', '01-01-2000 AD';
       '','','g_erm', 'i', '1';
       '','','g_abrc', 'i', '1';
       '','','g_obsrvr_pl', 'i', '0';
       '','','g_obsrvr_lon', 'i', '0';
       '','','g_obsrvr_lat', 'i', '0';
       '','','g_obsrvr_alt', 'i', '0';
       '','','g_lon_lat_alt', 'i', '0';      
       '','','g_fl_mat', 'i', '1';
       '','','g_fl_xls', 'i', '1';
       '','','g_fl_fig', 'i', '1';
       '','','g_fl_fig_save', 'i', '1';
       '','','g_fl_cf', 'i', '0'     
};
   
 ini_par = inifile('mean_LOD_from_365_solar_days.ini','read',keys);  
   n=numel(keys(:,1)); 
   for ii=1:n; 
    if ischar(cell2mat(ini_par(ii))); f_name= (horzcat(cell2mat(keys(ii,3)),'=''',cell2mat(ini_par(ii)),''';')); 
        elseif isnumeric(cell2mat(ini_par(ii)));  f_name= horzcat(cell2mat(keys(ii,3)),'=',num2str(cell2mat(ini_par(ii,1))),';');   
    end; eval(f_name); 
   end

%  Create and then hide the GUI as it is being constructed.
f = figure('Visible','off','Position',[380,500,450,385]);

htext_t_c = uicontrol('Style','text','String', ...
horzcat('Used ephemeris file is ', path_de(1:16) ,'...',path_de(numel(path_de)-12:numel(path_de)),filename_de), ...
          'Position',[20,365,350,15], 'HorizontalAlignment', 'Left');   
hbutton_select_dexxx = uicontrol('Style','pushbutton','String','Change', 'Position',[370,358,70,25], 'Callback',{@hbutton_select_dexxx_Callback});

% the epoch
htext_s_et = uicontrol('Style','edit','String',g_s_et, 'Position',[20,330,100,15],'HorizontalAlignment', 'Left'); 
uicontrol('Style','text','String','<', 'Position',[125,330,10,15]);
uicontrol('Style','text','String','ET, mm-dd-yyyy BC|AD', 'Position',[245,330,120,15]);
htext_e_et = uicontrol('Style','edit','String',g_e_et, 'Position',[140,330,100,15],'HorizontalAlignment', 'Left');

% the place
hpopup_obsrvr_places = uicontrol('Style','popupmenu', 'String',{g_obsrvr_places},...
          'Value', g_obsrvr_pl, 'Position',[20,310,100,15], 'Callback',{@hpopup_obsrvr_places_Callback});
uicontrol('Style','text','String','or', 'Position',[120,310,20,15]);
htext_obsrvr_lon = uicontrol('Style','edit','String',g_obsrvr_lon, 'Position',[140,310,30,15]);
htext_obsrvr_lat = uicontrol('Style','edit','String',g_obsrvr_lat, 'Position',[175,310,30,15]);
htext_obsrvr_alt = uicontrol('Style','edit','String',g_obsrvr_alt, 'Position',[210,310,30,15]);
hchbx_lon_lat_alt = uicontrol('Style','checkbox','String','(LON, LAT, ALT)', 'Position', [245,310,100,15], 'Callback',{@chbx_lon_lat_alt_Callback});
set(hchbx_lon_lat_alt,'Value',g_lon_lat_alt);
chbx_lon_lat_alt_Callback(hchbx_lon_lat_alt);
hpopup_obsrvr_places_Callback(hpopup_obsrvr_places);


htext_erm = uicontrol('Style','text','String','the Earth rotation model', 'Position',[290,250,150,15],'HorizontalAlignment', 'Left');
hpopup_erm = uicontrol('Style','popupmenu', 'String',{'rude IAU', 'precise ITRF93'},...
          'Value', g_erm, 'Position',[290,235,150,15], 'Callback',{@hpopup_erm_Callback});

htext_abrc = uicontrol('Style','text','String','Abberation correction','Position',[290,200,150,15],'HorizontalAlignment', 'Left');
hpopup_abrc = uicontrol('Style','popupmenu', 'String',{'none', 'LT+S'},...
          'Value', g_abrc, 'Position',[290,185,150,15], 'Callback',{@hpopup_abrc_Callback});


htext_s_p_o = uicontrol('Style','text','String','The start point on to Earth''s orbit is at', ...
              'Position',[20,250,250,15],'HorizontalAlignment', 'Left');
hpopup_s_p_o = uicontrol('Style','popupmenu',...
          'String',{'Aphelion', 'Perihelion', 'Equinox vernal', 'Solstice summer', 'Equinox autumn', 'Solstice winter'},...
          'Value', g_s_p_o, 'Position',[20,225,250,25], 'Callback',{@hpopup_s_p_o_Callback});
htext_s_p_d = uicontrol('Style','text','String','The start point in day period is at', 'Position',[20,200,150,15], ...
             'HorizontalAlignment', 'Left');
hpopup_s_p_d = uicontrol('Style','popupmenu', 'String',{'the event (aphelion, equinox','the noon (LST)','the midnight (LST)'},...
          'Value', g_s_p_d, 'Position',[20,175,200,25], 'Callback',{@hpopup_s_p_d_Callback});
      
htext_w_of_d = uicontrol('Style','text','String','The way of back determination of event', 'Position',[20,150,250,15], ...
           'HorizontalAlignment', 'Left');
hpopup_g_w_of_d = uicontrol('Style','popupmenu', 'String',{'by azimuth of sun in the topo frame','by longitude of center of Earth on to ecliptics', 'by longitude of center of Earth on to J2000 frame'},...
          'Value', g_w_of_d, 'Position',[20,125,250,25], 'Callback',{@hpopup_w_of_d_Callback});
      
hchbx_fl_fig = uicontrol('Style','checkbox','String','to the plot', 'Position', [325,85,115,15], 'Callback',{@chbx_fl_fig_Callback});
set(hchbx_fl_fig,'Value',g_fl_fig);
hchbx_fl_fig_save = uicontrol('Style','checkbox','String','to save', 'Position', [390,85,115,15], 'Callback',{@chbx_fl_fig_save_Callback});
set(hchbx_fl_fig_save,'Value',g_fl_fig_save);
hchbx_fl_xls = uicontrol('Style','checkbox','String','to array in file *.xls', 'Position', [325,70,115,15], 'Callback',{@chbx_fl_xls_Callback});
set(hchbx_fl_xls,'Value',g_fl_xls);
hchbx_fl_mat = uicontrol('Style','checkbox','String','to array in file *.mat', 'Position', [325,55,115,15], 'Callback',{@chbx_fl_mat_Callback});
set(hchbx_fl_mat,'Value',g_fl_mat);
hchbx_fl_cf = uicontrol('Style','checkbox','String','to the cftool', 'Position', [325,40,115,15], 'Callback',{@chbx_fl_cf_Callback});
set(hchbx_fl_cf,'Value',g_fl_cf);

   hbutton_start = uicontrol('Style','pushbutton','String','Run', 'Position',[290,10,70,25], 'Callback',{@startbutton_Callback});
   hbutton_end = uicontrol('Style','pushbutton','String','Exit', 'Position',[370,10,70,25], 'Callback',{@endbutton_Callback});

   set([f, htext_s_p_o, hpopup_s_p_o, htext_s_p_d, hpopup_s_p_d, htext_s_et, htext_e_et, ...
       hchbx_fl_fig, hchbx_fl_fig_save, hchbx_fl_mat, hchbx_fl_cf ...
       hpopup_obsrvr_places, htext_obsrvr_lon, htext_obsrvr_lat, htext_obsrvr_alt, htext_erm, hpopup_erm, htext_abrc, hpopup_abrc...
       htext_w_of_d,htext_w_of_d, htext_t_c, hbutton_select_dexxx, hbutton_start, hbutton_end ...
       ], 'Units','normalized');
   % Assign the GUI a name to appear in the window title.
   set(f,'Name','Calculation of length of mean solar day'); movegui(f,'center'); set(f,'Visible','on');
   
   function chbx_lon_lat_alt_Callback(hObject, eventdata, handles)
        if get(hObject,'Value') == 1; set(hpopup_obsrvr_places, 'Style','text');
             set(htext_obsrvr_lon,'Style','edit'); set(htext_obsrvr_lat,'Style','edit'); set(htext_obsrvr_alt,'Style','edit');    
         elseif get(hObject,'Value')==0; set(hpopup_obsrvr_places, 'Style','popupmenu');
             set(htext_obsrvr_lon,'Style','text'); set(htext_obsrvr_lat,'Style','text'); set(htext_obsrvr_alt,'Style','text');
         end 
   end
 function chbx_fl_mat_Callback(hObject, eventdata, handles)
 end
 function chbx_fl_xls_Callback(hObject, eventdata, handles)
 end
 function chbx_fl_fig_Callback(hObject, eventdata, handles)
 end
function chbx_fl_fig_save_Callback(hObject, eventdata, handles)
 end
 function chbx_fl_cf_Callback(hObject, eventdata, handles)
 end
 
   
   
     function hpopup_erm_Callback(source,eventdata) 
         % Determine the selected data set.
         str = get(source, 'String'); val = get(source,'Value');
         % Set current data to the selected data set.
         switch str{val};
         case 'rude IAU';           col_view = 1;
         case 'precise ITRF93';     col_view = 2;
         end
     end
 function hpopup_abrc_Callback(source,eventdata) 
         % Determine the selected data set.
         str = get(source, 'String'); val = get(source,'Value');
         % Set current data to the selected data set.
         switch str{val};
         case 'none';     col_view = 1;
         case 'lt+s';     col_view = 2;
         end
 end
 function hpopup_obsrvr_places_Callback(source,eventdata) 
         % Determine the selected data set.
         str = get(source, 'String'); val = get(source,'Value');
         [lat, lon, alt, ~, ~] = get_observer(deblank(str{val})); 
         set(htext_obsrvr_lon,'String',num2str(lon)); set(htext_obsrvr_lat,'String',num2str(lat));
         set(htext_obsrvr_alt,'String',num2str(alt));    
 end
     function hpopup_s_p_o_Callback(source,eventdata) 
         % Determine the selected data set.
         str = get(source, 'String'); val = get(source,'Value');
         % Set current data to the selected data set.
         switch str{val};
         case  'Aphelion';           col_view = 1;
         case  'Perihelion';        col_view = 2;
         case  'Equinox vernal';    col_view = 3;
         case  'Solstice summer';   col_view = 4;
         case  'Equinox autumn';    col_view = 5;
         case  'Solstice winter';   col_view = 6;
         end
   end
  function hpopup_s_p_d_Callback(source,eventdata) 
         % Determine the selected data set.
         str = get(source, 'String'); val = get(source,'Value');
         % Set current data to the selected data set.
         switch str{val};
         case 'at event (aphelion, equinox';    col_view = 1;
         case  'at noon LST';                   col_view = 2;
         case  'at midnight LST';               col_view = 3;
         end
  end
  function hpopup_w_of_d_Callback(source,eventdata) 
         % Determine the selected data set.
         str = get(source, 'String'); val = get(source,'Value');
         % Set current data to the selected data set.
         switch str{val};
         case 'by azimuth of sun in the topo frame';                col_view = 1;
         case 'by longitude of center of Earth on to ecliptics';    col_view = 2;
         case 'by longitude of center of Earth on to J2000 frame'; col_view = 3;
         end
  end
  function hbutton_select_dexxx_Callback(source,eventdata)
  cspice_unload( horzcat(path_de,filename_de));      
  [filename_de, path_de] = uigetfile('*.bsp','Select the ephemeris file', path_de);
  set(htext_t_c,'String',horzcat('Used ephemeris file is ', path_de(1:12) ,'...',path_de(numel(path_de)-12:numel(path_de)),filename_de));
  cspice_furnsh( horzcat(path_de,filename_de));      
  end

function write_ini_file()
  g_s_et=get(htext_s_et,'String');
  g_e_et=get(htext_e_et,'String');
  g_erm=get(hpopup_erm,'Value');
  g_abrc=get(hpopup_abrc,'Value');
  g_s_p_o=get(hpopup_s_p_o,'Value');
  g_s_p_d=get(hpopup_s_p_d,'Value');
  g_w_of_d=get(hpopup_g_w_of_d,'Value');
  g_obsrvr_pl=get(hpopup_obsrvr_places,'Value');
  g_obsrvr_lon=get(htext_obsrvr_lon,'String');
  g_obsrvr_lat=get(htext_obsrvr_lat,'String');
  g_obsrvr_alt=get(htext_obsrvr_alt,'String');
  g_lon_lat_alt=get(hchbx_lon_lat_alt,'Value');
  g_fl_mat=get(hchbx_fl_mat,'Value');
  g_fl_xls=get(hchbx_fl_xls,'Value');
  g_fl_fig=get(hchbx_fl_fig,'Value');
  g_fl_fig_save=get(hchbx_fl_fig_save,'Value');
  g_fl_cf=get(hchbx_fl_cf,'Value');
  
  
%   inifile('mean_LOD_from_365_solar_days.ini','new');
  inifile('mean_LOD_from_365_solar_days.ini','write',...
      {'','','path_de',path_de,'d:\Documents and Settings\user\ \MATLAB\astro\mice\data\';
       '','','filename_de',filename_de,'de422.bsp';
       '','','save_path',save_path,'..\';
       '','','g_s_p_o', g_s_p_o, '1';
       '','','g_s_p_d', g_s_p_d, '1';
       '','','g_w_of_d', g_w_of_d, '1';
       '','','g_s_et', g_s_et, '01-01-1970 AD';
       '','','g_e_et', g_e_et, '01-01-2000 AD';
       '','','g_erm', g_erm, '1';
       '','','g_abrc', g_abrc, '1';
       '','','g_obsrvr_pl', g_obsrvr_pl,'2';
       '','','g_obsrvr_lon', g_obsrvr_lon, '0';
       '','','g_obsrvr_lat', g_obsrvr_lat, '0';
       '','','g_obsrvr_alt', g_obsrvr_alt, '0';
       '','','g_lon_lat_alt', g_lon_lat_alt, '0';
       '','','g_fl_mat', g_fl_mat, '1';     
       '','','g_fl_xls', g_fl_xls, '1';     
       '','','g_fl_fig', g_fl_fig, '1'; 
       '','','g_fl_fig_save', g_fl_fig_save, '1'; 
       '','','g_fl_cf', g_fl_cf, '0'
},'plain');
  end

   

  function startbutton_Callback(source,eventdata)
      cspice_furnsh( horzcat(path_de,filename_de));  write_ini_file();
      s_p_o={'APHEL';'PERIH';'EQNXV';'SOLSM';'EQNXA';'SOLSW'}; s_p_d={'EXCT';'NOON';'MDNT'}; w_of_d={'AZMTH';'ECLIP';'J2000'};
      erm={'IAU_';'ITRF'}; abrc={'none';'LT&S'}; 

      TFMT='DD.MM.YYYY HR:MN:SC ERA::RND::MCAL::TDT'; TFMTH='HR:MN:SC.## ::RND::TDT' ; 
      MAXWIN  = 3000000; MAXIVL  = MAXWIN / 2; adjust=0.;
      % the path to output -  
      
      et1=cspice_str2et ( g_s_et ); et2=cspice_str2et ( g_e_et );  
      days =  (et2 - et1)/ cspice_spd; st_win = cspice_wninsd (et1 , et2);
      
      % SELECT precision model. For an accurate precision model of ER. Furnish earth_*_*.bpc
      if g_erm==1;  cspice_pcpool('TKFRAME_EARTH_FIXED_RELATIVE', 'IAU_EARTH' ); % a rude precision model of Earth rotation
      elseif g_erm==2;cspice_pcpool('TKFRAME_EARTH_FIXED_RELATIVE', 'ITRF93' ); end % an accurate 
      
                
      % ===============================================================================================================
      
      % SELECT aberration correction 
      if g_abrc==1; abcorr = 'NONE'; elseif g_abrc==2; abcorr = 'LT+S'; end 
      % ===============================================================================================================

      % SELECT beginning points of periods (aphelion, perohelion, vernal equinox, ... etc.)
      target  = 'SUN'; observer='399'; step    = 0.45 * cspice_tyear; nintvls = 300000; refval=0;% common parameters
      
      if g_s_p_o < 3; % aphelion, perohelion
          if g_s_p_o==1; relate  = 'LOCMAX'; elseif g_s_p_o==2; relate  = 'LOCMIN'; end  % 1. aphelions 2. perigelions
      res = cspice_gfdist( target, abcorr, observer, relate, refval, adjust, step,  nintvls, st_win);
      
      elseif g_s_p_o > 2 % equinoxes
       if g_s_p_o == 3; refval  = 0.; elseif g_s_p_o == 4; refval  = 90.; 
       elseif g_s_p_o == 5; refval  = 180.; elseif g_s_p_o == 6;  refval  = 270.; end
      frame = 'ECLIPJ2000'; crdsys =  'LATITUDINAL'   ; coord  = 'LONGITUDE' ; relate = '='; 
      res = cspice_gfposc ( target, frame, abcorr, observer, crdsys, coord, relate, refval ,  adjust, step, nintvls, st_win);
      end
      % ==============================================================================================================
      
      % TUNE beginning points of periods (the events, noons, midnights) 
      % the moments of the azimuths of the sun = 0
      if g_s_p_d>1; if g_s_p_d==2; refval = 0; elseif g_s_p_d==3; refval = 180; end
      
      % to wide moments to 1 day futher
      res=(reshape(res,2,[])'); res(:,2)= res (:,1) + 1.002 * cspice_spd; n1= numel(res(:,1)); 

      target = 'Sun'; crdsys = 'LATITUDINAL';coord = 'LONGITUDE'; relate='='; adjust=0; step = 0.2 * cspice_spd;
      
      if g_lon_lat_alt == 0; [observer, frame ]=set_observer(strtrim(g_obsrvr_places(g_obsrvr_pl,:))); 
      else [observer, frame ]=set_observer(str2double(g_obsrvr_lon),str2double(g_obsrvr_lat),str2double(g_obsrvr_alt)); end  
      
      for ii=1:n1 ; st_win_ii=[res(ii, 1), res(ii, 2)]';      
      res_t=cspice_gfposc( target,frame,abcorr,observer,crdsys,coord,relate,refval,adjust,step,nintvls,st_win_ii);
%       if numel(res_t)>2; fprintf('\n Found more the tunings than are events at %s!\n', cspice_timout( res_t(1), TFMT));end
      res(ii,1)=res_t(1); res(ii,2)=res_t(1);
      end
           
%       d_t= res_t(1)-res(1,1); % delta T between noon (midnight) and event (aphelion , perihelion, equinox, solstice)
%       fprintf ( '\ntune to %s %3.2f sec \nfor the 1st event: from %s to %s\n', cell2mat(s_p_d(g_s_p_d)), d_t, ...
%           cspice_timout(res (1,1), TFMT), cspice_timout( res_t(1), TFMT)) ;                                                     
%       
%       res(:,1)= res (:,1) + d_t - 0.1 * cspice_spd; res(:,2)= res (:,1) + d_t + 0.1 * cspice_spd; res=reshape(res',1,[])'; 
      % refining all the events to tune
%       res=cspice_gfposc( target,frame,abcorr,observer,crdsys,coord,relate,refval,adjust,step,nintvls,res);
      else  res=(reshape(res,2,[])');
      end
      % ==============================================================================================================
      
      res_s = res (:,1); res_s = res_s(1:numel(res_s)-1,1); % beginnings of periods
%       n2=numel(res(:,1)); if n1 ~= n2; fprintf('\n The result is not valid: not all events %s has their tune %s!!!\n', num2str(n1),num2str(n2));end
      res_e = res (:,2); res_e = res_e(2:numel(res_e),1); % ends of periond 
      res_l = res_e - res_s; % lenghts of periods between (not 365 days) events (aphelion , perihelion, equinox, solstice)
%       res_l_mid = res_l - cspice_tyear; 
      

      % SELECT the way to determinate the border of a period 
      if g_lon_lat_alt == 0; [observer, topo_frame ]=set_observer(strtrim(g_obsrvr_places(g_obsrvr_pl,:))); 
      else [observer, topo_frame ]=set_observer(str2double(g_obsrvr_lon),str2double(g_obsrvr_lat),str2double(g_obsrvr_alt)); end

      target = 'Sun'; crdsys = 'LATITUDINAL';coord = 'LONGITUDE'; relate='='; adjust=0; step = 0.22*cspice_spd;
      
      n=numel(res_e); res_365 = zeros(n,1);           
      st_win(1:numel(res_e),1) = res_s + 364.7 * cspice_spd; 
      st_win(1:numel(res_e),2) = res_s + 365.3 * cspice_spd; 
      for ii=1:numel(res_s);  st_win_ii=[st_win(ii, 1), st_win(ii, 2)]';
      if g_w_of_d==1; % by azimuth of sun in horizontal frame  
      [pos, ~] = cspice_spkpos ( target, res_s(ii), topo_frame, abcorr, observer); [~, az_sun, ~] = cspice_recrad(pos);           
      res = cspice_gfposc(target,topo_frame,abcorr,observer,crdsys,coord,relate,az_sun,adjust,step,nintvls,st_win_ii); 
      if numel(res)>2;
      for kk=1:2:numel(res); fprintf ( 'alarm! more than 1 azimuth per day %3.2f - %s\n', az_sun*cspice_dpr, cspice_timout( res(kk), 'DD.MM.YYYY ERA JCAL HR:MN ::RND::JCAL::UTC UTC')) ; end
      end
      
      % ==============================================================================================================                                                    
      elseif g_w_of_d==2
      % by angle between sun and 399 from observer in ecliptic plane
      [pos, ~] = cspice_spkpos ( target, res_s(ii), 'ECLIPJ2000', abcorr, observer); [~, lon, ~] = cspice_recrad(pos);
      res = cspice_gfposc(target,'ECLIPJ2000',abcorr,observer,crdsys,coord,relate,lon,adjust,step,nintvls,st_win_ii); 
      % ==============================================================================================================
      elseif g_w_of_d==3
      % by angle between sun and 399 from observer in ecliptic plane
      [pos, ~] = cspice_spkpos ( target, res_s(ii), 'J2000', abcorr, observer); [~, lon, ~] = cspice_recrad(pos);
      res = cspice_gfposc(target,'J2000',abcorr,observer,crdsys,coord,relate,lon,adjust,step,nintvls,st_win_ii); 
      end
      res_365(ii,1)=res(1); 
      end
      mean_365= (res_365 - res_s)/365; 
      % ==============================================================================================================
      
%       cftool(1:numel(mean_365),mean_365);
% output in XLS
[lat, lon, alt, sslat, sslon] = get_observer(deblank(observer)); 
f_place = horzcat(deblank(observer),'_',strtrim(sslat),'_',strtrim(sslon),'_ALT_',num2str(alt));
f_name=cell2mat(horzcat(filename_de(1:5),'_',s_p_o(g_s_p_o),'_',s_p_d(g_s_p_d),'_',w_of_d(g_w_of_d),'_',erm(g_erm),'_',abrc(g_abrc),'_',g_s_et,'_',g_e_et,'_',f_place));
f_name=regexprep(f_name, '-|\.|\x20|__', '_');

if g_fl_xls==1   % to output an XLS sheet
xlswrite(horzcat(save_path,f_name,'.xls'), mean_365); 
% fprintf(horzcat('the result saved in the Excel file ',...
%     regexptranslate('wildcard', save_path), regexptranslate('wildcard', f_name), '.xls' ));
end
if g_fl_mat==1   % to output an MatLab array in *.mat file
save(horzcat(save_path,f_name,'.mat'), 'mean_365');  %fprintf(horzcat('the result saved in the MAT file ', save_path,f_name,'.mat' ));
end

if g_fl_fig==1       
        f2=figure(2); 
        set(f2, 'Name', f_name); 
        plot(mean_365); title(f_name,'interpreter','none'); %,'YTickLabel',get(gca, 'YTick')
%         set(p1,'YTickLabelMode','manual'); 
        
       if g_fl_fig_save == 1
        saveas(f2, horzcat(f_name,'.png'), 'png'); 
        saveas(f2, horzcat(f_name,'.fig'), 'fig'); 
       end
     
end

if g_fl_cf==1
       f3=figure(3); plot(res_l); title(f_name); set(gca,'YTickLabelMode','manual','Name',f_name); YTick=get(gca, 'YTick');
       set(gca,'YTickLabel',YTick); 
end

end % WORK   =========================================================================================================
   
  function endbutton_Callback (source,eventdata)
  write_ini_file();
  close all
  cspice_kclear  
  clear all
  
  end

  end